#!/usr/bin/env python3
import os
import fileinput
import sys
import click
import aws_credential_process
import time
import configparser

"""
.config/systemd/user/aws_assume@.service

[Unit]
Description=Amazon Web Services token daemon

[Service]
Type=simple
ExecStart=/usr/bin/python3 -u %h/bin/_assume --rolearn='...%i...' --oath_slot=... --serialnumber=... --profile_name='...%i...'
Restart=on-failure

[Install]
WantedBy=default.target
"""

"""
.aws/credentials

[default]
aws_access_key_id = ...
aws_secret_access_key = ...

[...]
aws_access_key_id = ...
aws_secret_access_key = ...
aws_session_token = ...
"""


@click.command()
@click.option("--rolearn", required=True)
@click.option("--oath_slot", required=True)
@click.option("--serialnumber", required=True)
@click.option("--profile_name", required=True)
@click.option("--access-key-id", required=False)
@click.option("--secret-access-key", required=False)
@click.option("--mfa-session-duration", type=int)
@click.option("--credentials-section", default="default")
def main(
    rolearn,
    oath_slot,
    serialnumber,
    profile_name,
    access_key_id,
    secret_access_key,
    mfa_session_duration,
    credentials_section,
):
    invalid_token = None
    if not access_key_id:
        access_key_id, secret_access_key = aws_credential_process.get_credentials(credentials_section)

    if access_key_id is None:
        click.echo(
            "Missing access_key_id, please use --access-key-id or add to ~/.aws/credentials"
        )
        sys.exit(1)
    if secret_access_key is None:
        click.echo(
            "Missing secret_access_key, please use --secret-access-key or add to ~/.aws/credentials"
        )
        sys.exit(1)

    access_key = aws_credential_process.AWSCred(access_key_id, secret_access_key)

    while 1:
        mfa_session = None
        while mfa_session is None:
            mfa_session = aws_credential_process.get_mfa_session_cached(
                access_key, mfa_session_duration, serialnumber, oath_slot
            )
            time.sleep(1)

        assume_session = aws_credential_process.get_assume_session(
            access_key, mfa_session, rolearn, None
        )

        credentials_file = os.path.expanduser("~/.aws/credentials")
        # rotate credentials files
        for i in range(5, 0, -1):
            original = "{}.{}".format(credentials_file, i)
            if os.path.exists(original):
                os.rename(original, "{}.{}".format(credentials_file, i + 1))

        # update credentials file
        updated = {}
        profile = False
        for line in fileinput.input(credentials_file, inplace=True, backup=".1"):
            if profile and line[0] == "[":
                profile = False
            if line == "[{}]\n".format(profile_name):
                updated["profile"] = True
                profile = True
            if profile and line.startswith("aws_access_key_id"):
                updated["aws_access_key_id"] = True
                line = "aws_access_key_id = {}\n".format(
                    assume_session.awscred.access_key_id
                )
            if profile and line.startswith("aws_secret_access_key"):
                updated["aws_secret_access_key"] = True
                line = "aws_secret_access_key = {}\n".format(
                    assume_session.awscred.secret_access_key
                )
            if profile and line.startswith("aws_session_token"):
                updated["aws_session_token"] = True
                line = "aws_session_token = {}\n".format(assume_session.session_token)
            print(line, end="")

        if len(updated) == 0:
            print("Profile [{}] not found in ~/.aws/credentials".format(profile_name))
        if len(updated) < 4:
            for k in [
                "aws_access_key_id",
                "aws_secret_access_key",
                "aws_session_token",
            ]:
                if not k in updated:
                    print(
                        "{} not found in ~/.aws/credentials profile [{}]".format(
                            k, profile_name
                        )
                    )

        time.sleep(60 * 15)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['aws_assume']

package_data = \
{'': ['*']}

install_requires = \
['aws_credential_process>=0.1.0,<0.2.0', 'click==7.0']

entry_points = \
{'console_scripts': ['_assume = aws_assume._assume:main']}

setup_kwargs = {
    'name': 'aws-assume',
    'version': '0.2.2',
    'description': 'AWS session token refreshing daemon',
    'long_description': "AWS Assume daemon\n=================\n\nThis script automatically assumes every 15 minutes the specified role using a\nYubikey as MFA (multi factor authentication) and updates `~/.aws/credentials`.\n\nAs long as you've got your yubikey connected to your computer you'll never\nhave to enter a second factor authentication code for the aws cli. As other\ntools / libraries (boto3) use `~/.aws/credentials` as well you don't have to\nenter a token for these either.\n\nUsage\n=====\n\nYou can install aws_assume using pip (`pip install aws_assume`), I recommend\nto install aws_assume using poetry (`poetry install aws_assume`) or in a\nvirtualenv.\n\nYour `~/.aws/credentials` should contain your credentials and a profile with\nthe the keys `aws_access_key_id`, `aws_secret_access_key` and\n`aws_session_token`.\n\nFor example:\n\n`~/.aws/credentials`\n\n```ini\n[default]\naws_access_key_id = ...(your key id)...\naws_secret_access_key = ...(your access key)...\n\n[profile]\naws_access_key_id = ...(placeholder, can be anything)...\naws_secret_access_key = ...(placeholder, can be anything)...\naws_session_token = ...(placeholder, can be anything)...\n```\n\nYour `~/.aws/credentials` will be updated in place, only the specified profile\nsection should be touched (your comments will be safe).\n\nOlder versions are rotated up to 5 items.\n\nNext `_assume` should be started with the following arguments:\n\n```bash\n_assume --rolearn ... --oath_slot=... --serialnumber=... --profile_name=... --access-key-id=... --secret-access-key=... --mfa-session-duration=...\n```\n\nArgument                 | Description\n-------------------------|-------------------------------------\n`--rolearn`              | arn of the role you'd like to assume\n`--oath_slot`            | oath slot on your yubikey\n`--serialnumber`         | serial number of your MFA\n`--profile_name`         | profile used in `~/.aws/credentials`\n`--access-key-id`        | access key (as obtained from IAM console)\n`--secret-access-key`    | secret access key (as obtained from IAM console)\n`--mfa-session-duration` | duration (in seconds) for MFA session\n`--credentials-section`  | you can specify a different section than default in `~/.aws/credentials`\n\nYou should only run one `_assume` process per profile, I use systemd for\nstarting `_assume`, by using the following unit file:\n\n`~/.config/systemd/user/aws_assume@.service`\n\n```ini\n[Unit]\nDescription=Amazon Web Services token daemon\n\n[Service]\nType=simple\nExecStart=%h/bin/_assume --rolearn='...%i...' --oath_slot=... --serialnumber=... --profile_name='...%i...' --access-key-id='...' --secret-access-key='...'\nRestart=on-failure\n\n[Install]\nWantedBy=default.target\n```\n\nAnd reload systemd using `systemctl --user daemon-reload`, start `_assume` using\n`systemctl --user start aws_assume@...`\n\nIf you're not so fortunate to have systemd you can also use something like\n`supervisord` to start `_assume`.\n\n`~/supervisord.conf`\n\n```ini\n[supervisord]\n\n[supervisorctl]\nserverurl=unix:///home/user/supervisord.sock\n\n[unix_http_server]\nfile=/home/user/supervisord.sock\n\n[rpcinterface:supervisor]\nsupervisor.rpcinterface_factory = supervisor.rpcinterface:make_main_rpcinterface\n\n[program:assume-...]\ncommand=/home/user/bin/_assume --rolearn=... --oath_slot=... --serialnumber=... --profile_name=... --access-key-id=... --secret-access-key=...\nautorestart=true\n```\n\nStart supervisord using `supervisord -c supervisor.conf` and start assume using\n`supervisorctl -c supervisor.conf start assume-...`.\n",
    'author': 'Dick Marinus',
    'author_email': 'dick@mrns.nl',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/meeuw/aws_assume',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
